# example 1, trivariate joint Gaussian data

# definitions
n <- 1E6 # number of original samples
mu <- rep(0, 3); C <- matrix(c(0.35, 0.25, 0.5, 0.25, 0.4, 0.6, 0.5, 0.6, 1), nrow = 3)
x3cl <- c(0, 2) # conditioning planes in x3-direction
nb <- 1E5 # number of bootstrap samples
dx <- 0.6; np <- 100 # plot data range and number of plot points
color <- colorRampPalette(c("#FFFFFF","#E6E680","#E6BF1A",
                            "#E68000","#FF4026","#993380",
                            "#4D26BF","#262680","#000000")) # color ramp for plots

# generation original Gaussian samples
A <- eigen(C); B <- diag(A$values); A <- A$vectors
xs <- matrix(rnorm(3*n), nrow = 3)
xs <- A %*% (sqrt(B) %*% xs) + mu %*% t(rep(1,ncol(xs)))
pairs(t(xs[,seq(1, n, length.out = 1e4)]), labels = c("x1","x2","x3"), pch = ".")
d <- det.construct(xs, lb = 0, ub = 0, dtalim = 1E5) # det for bootstrap

# loop over conditioning planes
for (x3c in x3cl) {
   strg <- paste("p(x1,x2|x3=", toString(x3c), ")", sep = "") # plot title
   # grid for plotting
   x1m <- 0.5*x3c # <x1|x3=x3c>
   x2m <- 0.6*x3c # <x2|x3=x3c>
   px12cx3 <- function(x1,x2,x3) {10/pi*sqrt(5/3) *
      exp(-2/3 * (20*x1^2+50*x2^2+50*x1*(x2-x3)-85*x2*x3+38*x3^2))} # p(x1,x2|x3) exact
   pmax <- 1.5 * px12cx3(x1m,x2m,x3c) # maximum of p(x1,x2|x3)
   x1 <- 2*dx*(0:np)/np + x1m-dx; x2 <- 2*dx*(0:np)/np + x2m-dx # x1 & x2 grid
   xg <- rbind(rep(x1,np+1), rep(x2,each = np+1), x3c*rep(1,(np+1)^2)) # x1-x2-x3 grid

   # analytical reference
   px12cx3a <- matrix(rep(0,(np+1)^2), nrow = np+1)
   for (j in 1:(np+1)) {for (k in 1:(np+1)) {px12cx3a[j,k] <- px12cx3(x1[j],x2[k],x3c)}}
   filled.contour(x1, x2, px12cx3a, zlim = c(0,pmax), color = color)
   title(main = paste(strg,"reference"), xlab = "x1", ylab = "x2")

   # p(x1,x2|x3) from det estimator
   pxd <- matrix(det.query(d,xg), nrow = np+1) # p(x1,x2,x3)
   px3d <- det.query(det.construct(xs[3,,drop = FALSE], dtalim = 1E5), matrix(x3c)) # p(x3)
   filled.contour(x1, x2, pxd/px3d, zlim = c(0,pmax), color = color)
   title(main = paste(strg,"det"), xlab = "x1", ylab = "x2")

   # bootstrap conditional on x3 = x3c
   x12r <- det.rnd(nb, d, x3c, 3)
   # p(x1,x2|x3) from bootstrap data
   px12cx3d <- matrix(det.query(det.construct(x12r[1:2,]), xg[1:2,]), nrow = np+1)
   filled.contour(x1, x2, px12cx3d, zlim = c(0,pmax), color = color)
   title(main = paste(strg,"bootstrap"), xlab = "x1", ylab = "x2")
}
